package com.mortals.xhx.common.utils;

import com.mortals.xhx.module.record.web.ApplyLogController;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.filters.Watermark;
import net.coobird.thumbnailator.geometry.Positions;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

@Slf4j
public class WatermarkImgUtils {


    /**
     * @description
     * @param sourceImgPath 源图片路径
     * @param tarImgPath 保存的图片路径
     * @param waterMarkContent 水印内容
     * @param fileExt 图片格式
     * @return void
     */
    public static void addWatermark(String sourceImgPath, String tarImgPath, String waterMarkContent,String fileExt){
      //todo
        try {
            BufferedImage read = ImageIO.read(new File(sourceImgPath));
            BufferedImage bi = waterMarkByText(read.getWidth(),read.getHeight(),waterMarkContent,Color.RED,
                    new Font("宋体", Font.BOLD, 100),45.0,0.3f,sourceImgPath);
            ImageIO.write(bi, fileExt, new File(tarImgPath));
        } catch (IOException e) {
          log.error("添加水印失败",e);
        }



     /* return;


        Font font = new Font("宋体", Font.BOLD, 100);//水印字体，大小
        Color markContentColor = Color.red;//水印颜色
        Integer degree = 45;//设置水印文字的旋转角度
        float alpha = 0.3f;//设置水印透明度
        OutputStream outImgStream = null;
        try {
            File srcImgFile = new File(sourceImgPath);//得到文件
            Image srcImg = ImageIO.read(srcImgFile);//文件转化为图片
            int srcImgWidth = srcImg.getWidth(null);//获取图片的宽
            int srcImgHeight = srcImg.getHeight(null);//获取图片的高
            // 加水印
            BufferedImage bufImg = new BufferedImage(srcImgWidth, srcImgHeight, BufferedImage.TYPE_INT_RGB);
            Graphics2D g = bufImg.createGraphics();//得到画笔
            g.drawImage(srcImg, 0, 0, srcImgWidth, srcImgHeight, null);
            g.setColor(markContentColor); //设置水印颜色
            g.setFont(font);              //设置字体
            g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, alpha));//设置水印文字透明度
            if (null != degree) {
                g.rotate(Math.toRadians(degree));//设置水印旋转
            }
            JLabel label = new JLabel(waterMarkContent);
            FontMetrics metrics = label.getFontMetrics(font);
            int width = metrics.stringWidth(label.getText());//文字水印的宽
            int rowsNumber = srcImgHeight/width;// 图片的高  除以  文字水印的宽    ——> 打印的行数(以文字水印的宽为间隔)
            int columnsNumber = srcImgWidth/width;//图片的宽 除以 文字水印的宽   ——> 每行打印的列数(以文字水印的宽为间隔)
            //防止图片太小而文字水印太长，所以至少打印一次
            if(rowsNumber < 1){
                rowsNumber = 1;
            }
            if(columnsNumber < 1){
                columnsNumber = 1;
            }

            g.drawString(waterMarkContent, width-150 , - 280);

*//*            for(int j=0;j<rowsNumber;j++){
                boolean bool=false;
                for(int i=0;i<columnsNumber;i++){
                    g.drawString(waterMarkContent, i*width + j*width, -i*width + j*width);//画出水印,并设置水印位置
                bool=true;
                   break;
                }
                if (bool) break;
            }*//*
            g.dispose();// 释放资源
            // 输出图片
            log.info("生成水印图片路径：{}",tarImgPath);
            outImgStream = new FileOutputStream(tarImgPath);
            ImageIO.write(bufImg, fileExt, outImgStream);
        } catch (Exception e) {
            log.error("生成水印图片异常：",e);
        } finally{
            try {
                if(outImgStream != null){
                    outImgStream.flush();
                    outImgStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
                e.getMessage();
            }
        }*/
    }

    /**
     * 生成背景透明的 文字水印
     *
     * @param width     生成图片宽度
     * @param height    生成图片高度
     * @param text      水印文字
     * @param color     颜色对象
     * @param font      awt字体
     * @param degree    水印文字旋转角度
     * @param alpha     水印不透明度0f-1.0f
     * @param imagePath 图片地址
     */
    public static BufferedImage waterMarkByText(int width, int height, String text, Color color, Font font, Double degree, float alpha, String imagePath) {
        BufferedImage buffImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        // 得到画笔对象
        Graphics2D g2d = buffImg.createGraphics();
        // 增加下面的代码使得背景透明
        buffImg = g2d.getDeviceConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
        g2d.dispose();
        g2d = buffImg.createGraphics();

        // 设置对线段的锯齿状边缘处理
        g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

        //把源图片写入
        if (imagePath != null && !imagePath.equals("")) {
            try {
                Image srcImg = ImageIO.read(new File(imagePath));
                Image image = srcImg.getScaledInstance(srcImg.getWidth(null), srcImg.getHeight(null), Image.SCALE_SMOOTH);
                g2d.drawImage(image, 0, 0, null);
            } catch (IOException e) {
                log.error("生成水印图片异常：",e);
            }
        }

        // 设置水印旋转
        if (null != degree) {
            //注意rotate函数参数theta，为弧度制，故需用Math.toRadians转换一下
            //以矩形区域中央为圆心旋转
            g2d.rotate(Math.toRadians(degree), (double) buffImg.getWidth() / 2, (double) buffImg.getHeight() / 2);
        }

        // 设置颜色
        g2d.setColor(color);

        // 设置 Font
        g2d.setFont(font);

        // 设置透明度:1.0f为透明度 ，值从0-1.0，依次变得不透明
        g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));

        // 获取真实宽度
        float realWidth = getRealFontWidth(text);
        float fontSize = font.getSize();

        // 计算绘图偏移x、y，使得使得水印文字在图片中居中,x、y坐标是基于Graphics2D.rotate过后的坐标系
        float x = 0.5f * width - 0.5f * fontSize * realWidth;
        float y = 0.5f * height + 0.5f * fontSize;

        // 取绘制的字串宽度、高度中间点进行偏移，使得文字在图片坐标中居中
        g2d.drawString(text, x, y);
        // 释放资源
        g2d.dispose();
        return buffImg;
    }


    /**
     * 获取真实字符串宽度，ascii字符占用0.5，中文字符占用1.0
     *
     * @param text 文字
     * @return 宽度
     */
    private static float getRealFontWidth(String text) {
        int len = text.length();
        float width = 0f;
        for (int i = 0; i < len; i++) {
            if (text.charAt(i) < 256) {
                width += 0.5f;
            } else {
                width += 1.0f;
            }
        }
        return width;
    }


    public static void main(String[] args) throws IOException {

       WatermarkImgUtils.addWatermark("E:\\pic\\1724379763618.jpg", "E:\\pic\\1724379763618_water5.jpg", "广安市一业一证（照）一码", "png");

/*
        File watermarkFile = new File("watermark.png");
        BufferedImage watermarkImage = ImageIO.read(watermarkFile);

        Watermark watermark = new Watermark(Positions.CENTER, watermarkImage, 0.5f);

        Thumbnails.of("E:\\pic\\1724379763618.jpg").watermark(watermark).toFile("E:\\pic\\1724379763618_water2.jpg");
*/

//int width, int height, String text, Color color, Font font, Double degree, float alpha, String imagePath


/*        String sourceImage="E:\\pic\\1724379763618.jpg";
        BufferedImage read = ImageIO.read(new File(sourceImage));

        try {
            BufferedImage bi = waterMarkByText(read.getWidth(),read.getHeight(),"广安市一业一证（照）一码",Color.RED,
                    new Font("宋体", Font.BOLD, 100),45.0,0.3f,sourceImage);
            ImageIO.write(bi, "png", new File("E:\\pic\\1724379763618_water3.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }*/

    }
}
